# -*- coding: utf-8 -*-
"""
formatage des donnees et utilisation des gabarits Cheetah pour l'affichage
"""
from twisted.python import log
from nevow import loaders, tags as T
from nevow.url import URL
from os import path as osp
from os import unlink
from glob import glob

from ead2.frontend.web.lib.layout import TableLayout, SpecialTableLayout
from ead2.lib.error import TemplateError
from Cheetah.Template import Template as ChtTemplate
from Cheetah.NameMapper import NotFound as CheetahNotFound
from config import TEMPLATE_DIR
import urllib, commands, traceback

class CheetahTemplate(ChtTemplate):
    """personnalise et facilite la construction
    du template Cheetah
    """
    def __init__(self, filename, context):
        self.context = context
        builtins = {}
        ChtTemplate.__init__(self, file=filename, searchList=[context, builtins, {'is_defined':self.is_defined}])

    def is_defined(self, varname):
        """filtre permettant de ne pas lever d'exception au cas où
           la variable ne serait pas définie
        """
        return varname in self.context

def is_available(template):
    """:Returns:
    - True si le gabarit est disponible
    - False sinon
    """
    if osp.isfile(osp.join(TEMPLATE_DIR, template + '.tmpl')):
        return True
    return False

def write_template(templatename, content):
    """recupere un template s'il n'est pas disponible en local depuis le frontend web
    :templatename: nom du template
    :content: contenu du template
    """
    try:
        filename = osp.join(TEMPLATE_DIR,templatename + '.tmpl')
        fh = file(filename, 'w')
        fh.write(content)
        fh.close()
    except Exception, mess:
        fh.close()
        delete_template(templatename)
        raise Exception, "Erreur à l'écriture du template."

def delete_template(template):
    """ supprime un template non conforme """
    filename = osp.join(TEMPLATE_DIR,template + '.tmpl')
    unlink(filename)

def instanciate_template(template, context):
    """instancie un gabarit
    :Returns: la chaine instanciee
    """
    template_name = osp.join(TEMPLATE_DIR,template + '.tmpl')
    try:
        chtpl = CheetahTemplate(template_name, context['data']['content'])
        #chtpl = ChtTemplate(file=template_name, searchList=[context['data']['content'],{}])
        result = str(chtpl)
    except CheetahNotFound, err:
        # utiliser a ce moment la un template par default
        delete_template(template)
        raise TemplateError, "erreur dans l'instanciation du template %s" % template
    return str(result)

def compile_template(template):
    if not is_compiled(template):
        template_name = osp.join(TEMPLATE_DIR,template + '.tmpl')
        cmd = """/usr/bin/cheetah-compile %s"""%template_name
        a,b = commands.getstatusoutput(cmd)

def is_compiled(template):
    compiled_name = osp.join(TEMPLATE_DIR,template + '.py')
    if osp.isfile(compiled_name):
        return True
    return False

        
def format_action(result, proxy, magic_nb):
    """
    :proxy: le backend xml-rpc 
    :result: resultat de la sortie de l'action

    dans le resultat de l'action, un dictionnaire
    comprend:
    
    - une clef 'template' (le gabarit)
    - une clef 'data' (le resultat de l'action proprement dit)

    dans 'data' :
        - une clef 'content', qui est un contenu par defaut eventuellement
        - d'autres clefs qui peuvent etre utilisees comme variables de gabarit 
            (templates: widgets utilises)
    """
    # texte d'erreur par défaut
    error_text=""
    # unescape les caracteres speciaux apres le passage par xml-rpc
    result = urllib.unquote(result)
    # evalutation de la liste des sorties
    try:
        content = eval(result)
        data = content['data']
    except:
        # si ce qui est renvoye n'est pas normalise, c'est une chaine
        content = result
        data = content

    if type(data) == dict:
        if data.has_key('templates'):
            plugins = data['templates']
            for plug in plugins:
                if not is_available(plug):
                    content_plug = proxy.get_template(plug, magic_nb)[1]
                    write_template(plug, content_plug)
                compile_template(plug)

    if type(content) != str:
        if content.has_key('template'):
            try:
                # le template existe
                template = content['template']
                if is_available(template):
                    # le template est disponible
                    a = instanciate_template(template, content)
                    return T.xml(a)
                else:
                    # le template n'est pas disponible
                    try:
                        # tentative de recuperation du template
                        code, content_template = proxy.get_template(template, magic_nb)
                        write_template(template, content_template)
                        # instanciation cheetah
                        a = instanciate_template(template, content)
                        return T.xml(a)
                    except Exception, e:
                        # impossible de recuperer le template demande
                        raise Exception, e.message
            except Exception, e:
                log.err("Echec du templating (soit la forme soit le contenu)")
                traceback.print_exc()
                error_text = e.message

    return T.div[
                    T.p["Une erreur est survenue lors de la mise en forme de l'action demandée:"],
                    T.div[T.p["L'erreur suivante s'est produite: ", T.b[error_text]]],
                    T.a(href="javascript:Increase('div_contenu');", style='text_decoration:underline;')["Afficher le contenu reçu"],
                    T.div(_id='div_contenu', style="display:none;")[str(data)]
                ]

def put_to_nevow_xml(_list):
    """ rend des elements nevow depuis une liste de string"""
    result = []
    for i in _list:
        result.append(T.div[i])
    return result
