# -*- coding: utf-8 -*-
""" renvoit l'affichage d'une action """
from twisted.python import log
from nevow import loaders, tags as T, static, rend, inevow
from nevow.url import URL

import socket
from ead2.frontend.web.config import ENCODING
from ead2.frontend.web.lib import json
from ead2.frontend.web.gabarit import format_action
from ead2.frontend.web.config import ACTION_SOCKET_TIMEOUT

import traceback

class Action(rend.Page):
    """ gere les actions :
        :action_name: nom de l'action
        :menu: menu de description des actions disponibles
    """
    docFactory = loaders.xmlstr('''
            <div xmlns:nevow="http://nevow.com/ns/nevow/0.1">
                       <span nevow:render='action'/>
                       </div>
                       ''')

    def __init__(self, action_name, menu, ctx):
        self.ctx = ctx
        self.action = action_name
        self.actions = menu["actions"]
        self.menu = menu["menu"]

    def render_action(self, ctx, data):
        """ rend le formulaire pour une action """
        try:
            self.description = self.actions[self.action][0].encode(ENCODING)
            self.params = self.actions[self.action][1]
        except KeyError:
            return T.center()[T.img(src='/image/forbidden.png')]
        session = inevow.ISession(self.ctx)
        request = inevow.IRequest(self.ctx)
        url = str(URL.fromContext(self.ctx))
        key = None
        # si on revient d'un formulaire on evalue les donnees ecrites en Json
        if 'JSONstr' in request.args.keys():
            try:
                #result = eval(request.args['JSONstr'][0])# eval du retour javascript
                result = json.read(request.args['JSONstr'][0])
                key = result.keys()[0]
                return_res = result[key]
                request.args.pop('JSONstr')
            except:
                log.err( "Erreur de structure du param de type JSON dans la requête")
                key = None
                request.args.pop('JSONstr')
                return T.script["window.alert('Votre entrée de formulaire n\\'est pas conforme et ne peut être traitée.')"]
        # pour tout plugin on renvoit et les args de la requete afin qu il gere sa propre url
        params = {'request':request.args}
        # si on a des parametres JSON dans la requete on les renvoient
        if key is not None:
            params.update({key:return_res})
        # on renvoie l'url Ã  l'action
        params.update({'url':url})
        magic_nb = session.magic_number
        socket.setdefaulttimeout(ACTION_SOCKET_TIMEOUT)
        log.msg("####   Envoie au backend de la requête pour l'action %s " % self.action)
        #log.msg("Les valeurs envoyÃ©e au backend %s"%params)
        try:
            return_code, data = session.ead_server.execute_action(unicode(self.action), params, magic_nb)
            if return_code == 0:
                log.msg("++++ Retour d'action ok")
                return format_action(data, session.ead_server, magic_nb)
            else:
                log.err("---- Retour d'action négatif")
                return T.div()[T.img(src='image/warning.png'), T.br(),T.b["Le backend a renvoyé une erreur lors de l'éxécution de la requête %s" % data]]
        except socket.sslerror:
            log.err("---- Une requête trop longue n'a pas pu aboutir.")
            return T.div[
                             T.img(src='image/warning.png'),
                             T.br(),
                             T.b["Erreur rencontrée : aucune réponse du backend:"],T.br(),
                             "La connexion frontend-backend a été interrompue suite à un délai d'attente trop long.",
                             T.script()[
                            "window.alert('La requête est trop longue, le résultat ne peut être affiché');"
                            ]
                         ]
        except:
            traceback.print_exc()
            log.err("---- Une erreur a été rencontrée lors de l'éxécution de la requête")
            return T.div()[
                           T.img(src='image/warning.png'),
                           T.br(),
                           T.b["Une erreur a été rencontrée lors de la requête"]
                          ]
