# -*- coding: utf-8 -*-
import socket
from twisted.python import log
from ead2.frontend.config import config
from ead2.frontend.web.config import ACTION_SOCKET_TIMEOUT, TEST_SOCKET_TIMEOUT
from ead2.config.config import BACKEND_LISTEN_PORT, EADFILE_LISTEN_PORT, AUTH_SERVER_ADDR, AUTH_FORM_PORT,  AUTH_FORM_URL, ip_locale
from ead2.lib.libfrontend import ServerParser
from ead2.frontend.web.config import STATIC_DIR, SCRIPT_DIR, IMAGE_DIR, STYLE_DIR, TEMPLATE_DIR
from os import listdir, unlink, removedirs, remove
from os.path import join, isdir
from nevow import inevow
from nevow.url import URL
import commands, urllib2, md5

def servers():
    """ renvoit le parser de fichier gerant la liste des serveurs """
    SERVERS = ServerParser(BACKEND_LISTEN_PORT, config.SERVERS_FILE)
    return SERVERS

def get_dispo_styles():
    """ renvoit la liste des styles a choisir
        liste le rep style/choice
    """
    path = join(STYLE_DIR, 'choice')
    if isdir(path):
        files = [f for f in listdir(path) if ('css' in f and '~' not in f)]
        return files
    return []

def url_authform(current_url):
    """ renvoie l'url du formulaire en fonction de la provenance du client
    """
    if AUTH_SERVER_ADDR == ip_locale:
        parsed_url = list(urllib2.urlparse.urlsplit(str(current_url)))
        adress = urllib2.splitport(parsed_url[1])[0]
        return "https://%s:%s"%(adress, AUTH_FORM_PORT)
    else:
        return AUTH_FORM_URL

def sso_alive():
    """ teste si le serveur sso est disponible """
    try:
        socket.socket().connect((AUTH_SERVER_ADDR, int(AUTH_FORM_PORT)))
        return True
    except:
        return False

def _init_session( session, request):
    """
        initialise des paramètres
        de session pour l'ead
        (à purger)
    """
    if not hasattr(session, 'magic_number'):
        session.magic_number = None
    if not hasattr(session, 'language'):
        session.language='fr'
    # stockage des design disponibles
    if not hasattr(session, 'styles'):
        session.styles = get_dispo_styles()
    # design de l'EAD (style par défaut)
    if not hasattr(session, 'style'):
        style = request.getCookie('ead2style')
        if style != None :
            # style récupéré dans le cookie
            # TODO : style valide ? (à tester dans le render ?)
            session.style = style
        else:
            # style par défaut
            session.style = 'main1.css'
        # (re)mise à jour du cookie
        request.addCookie('ead2style', session.style, path='/', max_age=3600*24*365) #1an
   # menu de l'application
    if not hasattr(session, 'menu'):
        session.menu = None
   #identifiant du serveur en cour de traitement (numéro dans la liste)
    if not hasattr(session, 'module_name'):
        session.module_name = None
    if not hasattr(session, 'current_server'):
        session.current_server = None
   # gestion des actions en cour de traitement
    if not hasattr(session, 'current_action'):
        session.current_action = None
    if not hasattr(session, 'current_action_args'):
        session.current_action_args = None
   # variable pour signifier le retour des actions frontend (add_server ...
    if not hasattr(session, 'validate_error'):
        session.validate_error = None
    if not hasattr(session, 'validate_msg'):
        session.validate_msg = None
   #proxy xml-rpc pointant vers le backend
    if not hasattr(session, 'ead_server'):
        session.ead_server = None
    if not hasattr(session, 'role'):
        session.role = None
    return session

def purge_session_params(session):
    """
        purge les variables liées à la session en cour (pour le logout)
    """
    reset_session_params(session)
    session.magic_number = None
    session.role = None
    session.current_action = None
    session.current_action_args = None
    session.current_server = None

def reset_session_params(session):
    """ remet les parametres de session pour la redirection vers la page d'acceuil"""
    session.current_server = None
    session.menu = None
    session.ead_server = None
    session.validate_msg = None

def get_menu(ctx):
    """ gere la recuperation du menu  aupres du serveur"""
    session = inevow.ISession(ctx)
    request = inevow.IRequest(ctx)
    if session.menu is not None:
        return
    a, b = _get_menu(ctx)
    if a:
        log.msg("In get Menu")
        if b.has_key('role'):
            session.role = b['role']
            b.pop('role')
    # si l'on a le menu on le met dans la session
        session.menu = b
    else:
    # si l'on a pas le menu la premiere fois on retente
        a, b = _get_menu(ctx)
        if a:
            log.msg("In get Menu")
            session.role = b['role']
            b.pop('role')
            session.menu = b

def _get_menu(ctx):
    """ recupère le menu auprès du serveur """
    session = inevow.ISession(ctx)
    request = inevow.IRequest(ctx)
    if session.ead_server is not None:
#        magic_number = request.getCookie('magic')
        magic_number = session.magic_number
        if magic_number:
            try:
                socket.setdefaulttimeout(ACTION_SOCKET_TIMEOUT)
                code, data = session.ead_server.get_menu(magic_number)
                if data['menu'] == {}:
                    log.err("on nous rend un menu vide")
                    return None, ()
            except:
                log.err("erreur server")
                return None, ()
            if code in (3,4):
                log.err("erreur : ")
                log.err(code)
                # FIXME : redirection
                return None, ()
            if code != 0:
                log.err("Erreur Menu %d: %s"%(code, data))
                return None, ()
            try:
                menu = data
                return True,menu
            except:
                log.err("erreur au chargement du menu retour à lapage d'accueil")
                return None, ()
    log.err( "soit pas de serveur, soit pas de MN")
    return None, ()

#### UTILITAIRES DE TELECHARGEMENT DE FICHIER
def clean_datas():
    """
        lancer à chaque démarrage
        supprime les repertoires contenant les fichiers web telecharges:
            /image/<nom du module>/
            /style/<nom du module>/
            /script/<nom du module>/
        et supprime les templates deja downloader.
    """
    modules = ['amon', 'horus', 'sphynx', 'scribe', 'zephir', 'amonecole', 'eclair']
    for module_name in modules:
        script_dir = join(SCRIPT_DIR, module_name)
        style_dir = join(STYLE_DIR, module_name)
        image_dir = join(IMAGE_DIR, module_name)
        if isdir(script_dir):
            for i in listdir(script_dir):
                name = join(script_dir, i)
                remove(name)
            removedirs(script_dir)
        if isdir(style_dir):
            for i in listdir(style_dir):
                name = join(style_dir, i)
                remove(name)
            removedirs(style_dir)
        if isdir(image_dir):
            for i in listdir(image_dir):
                name = join(image_dir, i)
                remove(name)
            removedirs(image_dir)
        if isdir(TEMPLATE_DIR):
            for i in listdir(TEMPLATE_DIR):
                name = join(TEMPLATE_DIR, i)
                if not isdir(name):
                    remove(name)
                init = join(TEMPLATE_DIR, '__init__.py')
                file(init, 'w').write('')
    return True

def initialize_ead_files(session, request):
    """ initialise l'interface en chargeant les fichiers
        à inclure pour l'ead du serveur courant
        :session: objet session de nevow
        :session.ead_server: server en cour de traitement
    """
#    magic = request.getCookie('magic')
    magic = session.magic_number
    if session.ead_server is None: return
    code, module_name = session.ead_server.get_module_name(magic)

    if module_name == 'amonecole':
        module_name = 'amon'
    # recuperation des chemins d'import lies au module consulte
    script_dir = join(SCRIPT_DIR, module_name)
    style_dir = join(STYLE_DIR, module_name)
    image_dir = join(IMAGE_DIR, module_name)

    if module_name not in ['horus', 'scribe', 'amon', 'eclair']:
        session.module_name = None
        return
    else:
        session.module_name = module_name
    # listing des fichiers presents sur le frontend
    if isdir(script_dir):
        act_scripts = str(listdir(script_dir))
    else:
        commands.getstatusoutput('mkdir %s'%script_dir)
        act_scripts = "[]"
    if isdir(style_dir):
        act_styles = str(listdir(style_dir))
    else:
        commands.getstatusoutput('mkdir %s'%style_dir)
        act_styles = "[]"
    if isdir(image_dir):
        act_images = str(listdir(image_dir))
    else:
        commands.getstatusoutput('mkdir %s'%image_dir)
        act_images = "[]"
    # appel du backend pour lister les fichiers manquant
    call = {'script':act_scripts,
            'style':act_styles,
            'image':act_images}
    code, result = session.ead_server.execute_action(unicode('update_ead'), call, magic)
    if result == '{}':
        return
    try:
        result = eval(result)
    except:
        return
    url, port, comment, key = servers().get_server(session.current_server)
    url = URL.fromString(url)
    ead_file_url = url.secure(False)
    # recuperation des fichiers aupres du serveur http
    get_files(result['style'], 'style', style_dir, ead_file_url)
    get_files(result['script'], 'script', script_dir, ead_file_url)
    get_files(result['image'], 'image', image_dir, ead_file_url)

def get_files(files, keyword, typed_path, ead_file_url):
    http_path = "%s:%s"%(ead_file_url, EADFILE_LISTEN_PORT)
    for fil in files:
        from_path = join(join(http_path, keyword), fil)
        md5file = fil + '.md5'
        md5path = join(http_path, join('md5',join(keyword, md5file)))
        dest_path = join(typed_path, fil)
        get_file(from_path, md5path, dest_path)

def get_file(from_path, md5_path, dest_path):
    """telecharge "fich" et "fich.md5" sur serveur
        calcul le checksum de fich et le compare a celui contenu dans "fich.md5"
        :from_path: chemin vers le fichier a telecharger
        :md5_path: chemin vers le fichier md5
        :dest_path: chemin du fichier de destination
    """
    try:
        dest5 = dest_path + '.md5'
        log.msg('Telechargement de "%s"'%dest_path)
        log.msg('Telechargement depuis : %s'%from_path)
        file(dest_path, 'wb').write(urllib2.urlopen(from_path).read())
        file(dest5, 'wb').write(urllib2.urlopen(md5_path).read())
    except Exception, e:
        log.err("erreur")
        log.err(e)
        return None
    try:
        if check_sum(dest_path, dest5):
            # on supprime le fichier md5
            unlink(dest5)
            return dest_path
        else: return None
    except Exception, e:
        log.err("erreur md5 :")
        log.err(e )
        return None

def check_sum(fich, sumfich):
    """recupere le checksum contenu dans sumfich,
        calcule le checksum de fich
        et compare les deux
    """
    orig_sum = file(sumfich, 'r').readlines()[0].strip().split()[0].strip()
    dest_sum = md5.md5(file(fich,'rb').read()).hexdigest().strip()
    if orig_sum == dest_sum:
        return True
    else:
        return None
